@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -53,11 +53,18 @@ enum rspamd_http_priv_flags {
 	RSPAMD_HTTP_CONN_FLAG_PROXY = 1u << 5u,
 	RSPAMD_HTTP_CONN_FLAG_PROXY_REQUEST = 1u << 6u,
 	RSPAMD_HTTP_CONN_OWN_SOCKET = 1u << 7u,
+	RSPAMD_HTTP_CONN_FLAG_EARLY_RESPONSE = 1u << 8u, /* Server sent early response during write */
 };
 
 #define IS_CONN_ENCRYPTED(c) ((c)->flags & RSPAMD_HTTP_CONN_FLAG_ENCRYPTED)
 #define IS_CONN_RESETED(c) ((c)->flags & RSPAMD_HTTP_CONN_FLAG_RESETED)
 
+static gssize rspamd_http_try_read(int fd,
+								   struct rspamd_http_connection *conn,
+								   struct rspamd_http_connection_private *priv,
+								   struct _rspamd_http_privbuf *pbuf,
+								   const char **buf_ptr);
+
 struct rspamd_http_connection_private {
 	struct rspamd_http_context *ctx;
 	struct rspamd_ssl_connection *ssl;
@@ -69,13 +76,27 @@ struct rspamd_http_connection_private {
 	struct http_parser parser;
 	struct http_parser_settings parser_cb;
 	struct rspamd_io_ev ev;
-	ev_tstamp timeout;
+	ev_tstamp timeout; /* legacy/global timeout (fallback) */
+	/* Staged timeouts (seconds); 0 means use ctx/defaults */
+	ev_tstamp connect_timeout;
+	ev_tstamp ssl_timeout;
+	ev_tstamp write_timeout;
+	ev_tstamp read_timeout;
 	struct rspamd_http_message *msg;
 	struct iovec *out;
 	unsigned int outlen;
 	enum rspamd_http_priv_flags flags;
 	gsize wr_pos;
 	gsize wr_total;
+	/* Keepalive tuning and telemetry */
+	double created_ts;                  /* when connection object was created */
+	double last_used_ts;                /* last time returned to pool */
+	unsigned int reuse_count;           /* number of reuses from keepalive */
+	double ka_ttl_override;             /* per-request TTL */
+	double ka_idle_override;            /* per-request idle timeout */
+	unsigned int ka_max_reuse_override; /* per-request max reuse */
+	/* Internal state */
+	bool first_write_done;
 };
 
 static const rspamd_ftok_t key_header = {
@@ -731,11 +752,11 @@ rspamd_http_simple_client_helper(struct rspamd_http_connection *conn)
 
 	if (conn->opts & RSPAMD_HTTP_CLIENT_SHARED) {
 		rspamd_http_connection_read_message_shared(conn, conn->ud,
-												   conn->priv->timeout);
+												   (priv->read_timeout > 0 ? priv->read_timeout : conn->priv->timeout));
 	}
 	else {
 		rspamd_http_connection_read_message(conn, conn->ud,
-											conn->priv->timeout);
+											(priv->read_timeout > 0 ? priv->read_timeout : conn->priv->timeout));
 	}
 
 	if (priv->msg) {
@@ -812,8 +833,43 @@ rspamd_http_write_helper(struct rspamd_http_connection *conn)
 	}
 
 	if (r == -1) {
+		int saved_errno = errno;
+
 		if (!priv->ssl) {
-			err = g_error_new(HTTP_ERROR, 500, "IO write error: %s", strerror(errno));
+			/* Check if server sent an early response (e.g., 413) */
+			if (saved_errno == EPIPE || saved_errno == ECONNRESET ||
+				saved_errno == ENOTCONN) {
+				struct _rspamd_http_privbuf *pbuf = priv->buf;
+				const char *d;
+				gssize read_res;
+
+				REF_RETAIN(pbuf);
+				read_res = rspamd_http_try_read(conn->fd, conn, priv, pbuf, &d);
+
+				if (read_res > 0) {
+					msg_debug("got early server response (%z bytes) during write, processing",
+							  read_res);
+
+					if (http_parser_execute(&priv->parser, &priv->parser_cb,
+											d, read_res) != (size_t) read_res ||
+						priv->parser.http_errno != 0) {
+						err = g_error_new(HTTP_ERROR, 400,
+										  "HTTP parser error on early response: %s",
+										  http_errno_description(priv->parser.http_errno));
+						rspamd_http_connection_ref(conn);
+						conn->error_handler(conn, err);
+						rspamd_http_connection_unref(conn);
+						g_error_free(err);
+					}
+
+					REF_RELEASE(pbuf);
+					return;
+				}
+
+				REF_RELEASE(pbuf);
+			}
+
+			err = g_error_new(HTTP_ERROR, 500, "IO write error: %s", strerror(saved_errno));
 			rspamd_http_connection_ref(conn);
 			conn->error_handler(conn, err);
 			rspamd_http_connection_unref(conn);
@@ -852,8 +908,15 @@ call_finish_handler:
 		rspamd_http_connection_unref(conn);
 	}
 	else {
-		/* Plan read message */
-		rspamd_http_simple_client_helper(conn);
+		if (priv->flags & RSPAMD_HTTP_CONN_FLAG_EARLY_RESPONSE) {
+			/* Early response already being processed */
+			rspamd_ev_watcher_reschedule(priv->ctx->event_loop, &priv->ev, EV_READ);
+		}
+		else {
+			/* Plan read message */
+			priv->first_write_done = true;
+			rspamd_http_simple_client_helper(conn);
+		}
 	}
 }
 
@@ -939,7 +1002,17 @@ rspamd_http_event_handler(int fd, short what, gpointer ud)
 	REF_RETAIN(pbuf);
 	rspamd_http_connection_ref(conn);
 
-	if (what == EV_READ) {
+	if (what & EV_READ) {
+		/* Check for early response while still sending request (client only) */
+		if (conn->type == RSPAMD_HTTP_CLIENT &&
+			priv->wr_pos < priv->wr_total && priv->wr_total > 0) {
+			msg_debug("received early server response while still writing request "
+					  "(sent %uz of %uz bytes)",
+					  priv->wr_pos, priv->wr_total);
+			priv->wr_pos = priv->wr_total;
+			priv->flags |= RSPAMD_HTTP_CONN_FLAG_EARLY_RESPONSE;
+		}
+
 		r = rspamd_http_try_read(fd, conn, priv, pbuf, &d);
 
 		if (r > 0) {
@@ -1023,7 +1096,7 @@ rspamd_http_event_handler(int fd, short what, gpointer ud)
 			return;
 		}
 	}
-	else if (what == EV_TIMEOUT) {
+	else if (what & EV_TIMEOUT) {
 		if (!priv->ssl) {
 			/* Let's try to read from the socket first */
 			r = rspamd_http_try_read(fd, conn, priv, pbuf, &d);
@@ -1071,7 +1144,7 @@ rspamd_http_event_handler(int fd, short what, gpointer ud)
 			return;
 		}
 	}
-	else if (what == EV_WRITE) {
+	else if (what & EV_WRITE) {
 		rspamd_http_write_helper(conn);
 	}
 
@@ -1130,6 +1203,18 @@ rspamd_http_connection_new_common(struct rspamd_http_context *ctx,
 	priv = g_malloc0(sizeof(struct rspamd_http_connection_private));
 	conn->priv = priv;
 	priv->ctx = ctx;
+	/* Initialize staged timeouts and keepalive telemetry */
+	priv->connect_timeout = ctx->config.connect_timeout;
+	priv->ssl_timeout = ctx->config.ssl_timeout;
+	priv->write_timeout = ctx->config.write_timeout;
+	priv->read_timeout = ctx->config.read_timeout;
+	priv->created_ts = (ctx->event_loop ? ev_now(ctx->event_loop) : rspamd_get_ticks(FALSE));
+	priv->last_used_ts = 0;
+	priv->reuse_count = 0;
+	priv->ka_ttl_override = 0;
+	priv->ka_idle_override = 0;
+	priv->ka_max_reuse_override = 0;
+	priv->first_write_done = false;
 	priv->flags = priv_flags;
 
 	if (type == RSPAMD_HTTP_SERVER) {
@@ -1513,7 +1598,8 @@ rspamd_http_connection_read_message_common(struct rspamd_http_connection *conn,
 		priv->flags |= RSPAMD_HTTP_CONN_FLAG_ENCRYPTED;
 	}
 
-	priv->timeout = timeout;
+	/* Use read-stage timeout override if set; else fallback */
+	priv->timeout = (priv->read_timeout > 0 ? priv->read_timeout : timeout);
 	priv->header = NULL;
 	priv->buf = g_malloc0(sizeof(*priv->buf));
 	REF_INIT_RETAIN(priv->buf, rspamd_http_privbuf_dtor);
@@ -1670,7 +1756,22 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 {
 	char datebuf[64];
 	int meth_len = 0;
-	const char *conn_type = "close";
+	const char *server_conn_header, *client_conn_header;
+
+	/* Set up connection header strings based on flags and connection type */
+	if (msg->flags & RSPAMD_HTTP_FLAG_HAS_CONNECTION_HEADER) {
+		server_conn_header = "";
+		client_conn_header = "";
+	}
+	else {
+		server_conn_header = "Connection: close\r\n";
+		if (conn->opts & RSPAMD_HTTP_CLIENT_KEEP_ALIVE) {
+			client_conn_header = "Connection: keep-alive\r\n";
+		}
+		else {
+			client_conn_header = "Connection: close\r\n";
+		}
+	}
 
 	if (conn->type == RSPAMD_HTTP_SERVER) {
 		/* Format reply */
@@ -1712,12 +1813,14 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 					meth_len =
 						rspamd_snprintf(repbuf, replen,
 										"HTTP/1.1 %d %T\r\n"
-										"Connection: close\r\n"
+										"%s"
 										"Server: %s\r\n"
 										"Date: %s\r\n"
 										"Content-Length: %z\r\n"
 										"Content-Type: %s", /* NO \r\n at the end ! */
-										msg->code, &status, priv->ctx->config.server_hdr,
+										msg->code, &status,
+										server_conn_header,
+										priv->ctx->config.server_hdr,
 										datebuf,
 										bodylen, mime_type);
 				}
@@ -1725,11 +1828,13 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 					meth_len =
 						rspamd_snprintf(repbuf, replen,
 										"HTTP/1.1 %d %T\r\n"
-										"Connection: close\r\n"
+										"%s"
 										"Server: %s\r\n"
 										"Date: %s\r\n"
 										"Content-Length: %z", /* NO \r\n at the end ! */
-										msg->code, &status, priv->ctx->config.server_hdr,
+										msg->code, &status,
+										server_conn_header,
+										priv->ctx->config.server_hdr,
 										datebuf,
 										bodylen);
 				}
@@ -1737,11 +1842,12 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 				/* External reply */
 				rspamd_printf_fstring(buf,
 									  "HTTP/1.1 200 OK\r\n"
-									  "Connection: close\r\n"
+									  "%s"
 									  "Server: %s\r\n"
 									  "Date: %s\r\n"
 									  "Content-Length: %z\r\n"
 									  "Content-Type: application/octet-stream\r\n",
+									  server_conn_header,
 									  priv->ctx->config.server_hdr,
 									  datebuf, enclen);
 			}
@@ -1750,12 +1856,14 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 					meth_len =
 						rspamd_printf_fstring(buf,
 											  "HTTP/1.1 %d %T\r\n"
-											  "Connection: close\r\n"
+											  "%s"
 											  "Server: %s\r\n"
 											  "Date: %s\r\n"
 											  "Content-Length: %z\r\n"
 											  "Content-Type: %s\r\n",
-											  msg->code, &status, priv->ctx->config.server_hdr,
+											  msg->code, &status,
+											  server_conn_header,
+											  priv->ctx->config.server_hdr,
 											  datebuf,
 											  bodylen, mime_type);
 				}
@@ -1763,11 +1871,13 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 					meth_len =
 						rspamd_printf_fstring(buf,
 											  "HTTP/1.1 %d %T\r\n"
-											  "Connection: close\r\n"
+											  "%s"
 											  "Server: %s\r\n"
 											  "Date: %s\r\n"
 											  "Content-Length: %z\r\n",
-											  msg->code, &status, priv->ctx->config.server_hdr,
+											  msg->code, &status,
+											  server_conn_header,
+											  priv->ctx->config.server_hdr,
 											  datebuf,
 											  bodylen);
 				}
@@ -1804,10 +1914,6 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 	else {
 
 		/* Client request */
-		if (conn->opts & RSPAMD_HTTP_CLIENT_KEEP_ALIVE) {
-			conn_type = "keep-alive";
-		}
-
 		/* Format request */
 		enclen += RSPAMD_FSTRING_LEN(msg->url) +
 				  strlen(http_method_str(msg->method)) + 1;
@@ -1819,21 +1925,21 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 									  "%s %s HTTP/1.0\r\n"
 									  "Content-Length: %z\r\n"
 									  "Content-Type: application/octet-stream\r\n"
-									  "Connection: %s\r\n",
+									  "%s",
 									  "POST",
 									  "/post",
 									  enclen,
-									  conn_type);
+									  client_conn_header);
 			}
 			else {
 				rspamd_printf_fstring(buf,
 									  "%s %V HTTP/1.0\r\n"
 									  "Content-Length: %z\r\n"
-									  "Connection: %s\r\n",
+									  "%s",
 									  http_method_str(msg->method),
 									  msg->url,
 									  bodylen,
-									  conn_type);
+									  client_conn_header);
 
 				if (bodylen > 0) {
 					if (mime_type == NULL) {
@@ -1857,26 +1963,26 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 				if (rspamd_http_message_is_standard_port(msg)) {
 					rspamd_printf_fstring(buf,
 										  "%s %s HTTP/1.1\r\n"
-										  "Connection: %s\r\n"
+										  "%s"
 										  "Host: %s\r\n"
 										  "Content-Length: %z\r\n"
 										  "Content-Type: application/octet-stream\r\n",
 										  "POST",
 										  "/post",
-										  conn_type,
+										  client_conn_header,
 										  host,
 										  enclen);
 				}
 				else {
 					rspamd_printf_fstring(buf,
 										  "%s %s HTTP/1.1\r\n"
-										  "Connection: %s\r\n"
+										  "%s"
 										  "Host: %s:%d\r\n"
 										  "Content-Length: %z\r\n"
 										  "Content-Type: application/octet-stream\r\n",
 										  "POST",
 										  "/post",
-										  conn_type,
+										  client_conn_header,
 										  host,
 										  msg->port,
 										  enclen);
@@ -1888,21 +1994,21 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 					if ((msg->flags & RSPAMD_HTTP_FLAG_HAS_HOST_HEADER)) {
 						rspamd_printf_fstring(buf,
 											  "%s %s://%s:%d/%V HTTP/1.1\r\n"
-											  "Connection: %s\r\n"
+											  "%s"
 											  "Content-Length: %z\r\n",
 											  http_method_str(msg->method),
 											  (conn->opts & RSPAMD_HTTP_CLIENT_SSL) ? "https" : "http",
 											  host,
 											  msg->port,
 											  msg->url,
-											  conn_type,
+											  client_conn_header,
 											  bodylen);
 					}
 					else {
 						if (rspamd_http_message_is_standard_port(msg)) {
 							rspamd_printf_fstring(buf,
 												  "%s %s://%s:%d/%V HTTP/1.1\r\n"
-												  "Connection: %s\r\n"
+												  "%s"
 												  "Host: %s\r\n"
 												  "Content-Length: %z\r\n",
 												  http_method_str(msg->method),
@@ -1910,14 +2016,14 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 												  host,
 												  msg->port,
 												  msg->url,
-												  conn_type,
+												  client_conn_header,
 												  host,
 												  bodylen);
 						}
 						else {
 							rspamd_printf_fstring(buf,
 												  "%s %s://%s:%d/%V HTTP/1.1\r\n"
-												  "Connection: %s\r\n"
+												  "%s"
 												  "Host: %s:%d\r\n"
 												  "Content-Length: %z\r\n",
 												  http_method_str(msg->method),
@@ -1925,7 +2031,7 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 												  host,
 												  msg->port,
 												  msg->url,
-												  conn_type,
+												  client_conn_header,
 												  host,
 												  msg->port,
 												  bodylen);
@@ -1937,35 +2043,35 @@ int rspamd_http_message_write_header(const char *mime_type, gboolean encrypted,
 					if ((msg->flags & RSPAMD_HTTP_FLAG_HAS_HOST_HEADER)) {
 						rspamd_printf_fstring(buf,
 											  "%s %V HTTP/1.1\r\n"
-											  "Connection: %s\r\n"
+											  "%s"
 											  "Content-Length: %z\r\n",
 											  http_method_str(msg->method),
 											  msg->url,
-											  conn_type,
+											  client_conn_header,
 											  bodylen);
 					}
 					else {
 						if (rspamd_http_message_is_standard_port(msg)) {
 							rspamd_printf_fstring(buf,
 												  "%s %V HTTP/1.1\r\n"
-												  "Connection: %s\r\n"
+												  "%s"
 												  "Host: %s\r\n"
 												  "Content-Length: %z\r\n",
 												  http_method_str(msg->method),
 												  msg->url,
-												  conn_type,
+												  client_conn_header,
 												  host,
 												  bodylen);
 						}
 						else {
 							rspamd_printf_fstring(buf,
 												  "%s %V HTTP/1.1\r\n"
-												  "Connection: %s\r\n"
+												  "%s"
 												  "Host: %s:%d\r\n"
 												  "Content-Length: %z\r\n",
 												  http_method_str(msg->method),
 												  msg->url,
-												  conn_type,
+												  client_conn_header,
 												  host,
 												  msg->port,
 												  bodylen);
@@ -2025,7 +2131,8 @@ rspamd_http_connection_write_message_common(struct rspamd_http_connection *conn,
 
 	conn->ud = ud;
 	priv->msg = msg;
-	priv->timeout = timeout;
+	/* Use write-stage timeout override if set */
+	priv->timeout = (priv->write_timeout > 0 ? priv->write_timeout : timeout);
 
 	priv->header = NULL;
 	priv->buf = g_malloc0(sizeof(*priv->buf));
@@ -2367,8 +2474,10 @@ if (conn->opts & RSPAMD_HTTP_CLIENT_SSL) {
 												  conn->log_tag);
 			g_assert(priv->ssl != NULL);
 
+			/* Use ssl_timeout for handshake if provided */
+			ev_tstamp ssl_to = (priv->ssl_timeout > 0 ? priv->ssl_timeout : (priv->connect_timeout > 0 ? priv->connect_timeout : priv->timeout));
 			if (!rspamd_ssl_connect_fd(priv->ssl, conn->fd, host, &priv->ev,
-									   priv->timeout, rspamd_http_event_handler,
+									   ssl_to, rspamd_http_event_handler,
 									   rspamd_http_ssl_err_handler, conn)) {
 
 				err = g_error_new(HTTP_ERROR, 400,
@@ -2388,14 +2497,18 @@ if (conn->opts & RSPAMD_HTTP_CLIENT_SSL) {
 												   rspamd_http_event_handler,
 												   rspamd_http_ssl_err_handler,
 												   conn,
-												   EV_WRITE);
+												   EV_WRITE | EV_READ);
 		}
 	}
 }
 else {
-	rspamd_ev_watcher_init(&priv->ev, conn->fd, EV_WRITE,
+	/* Watch for READ too on client to detect early server responses */
+	short ev_flags = (conn->type == RSPAMD_HTTP_CLIENT) ? (EV_WRITE | EV_READ) : EV_WRITE;
+	rspamd_ev_watcher_init(&priv->ev, conn->fd, ev_flags,
 						   rspamd_http_event_handler, conn);
-	rspamd_ev_watcher_start(priv->ctx->event_loop, &priv->ev, priv->timeout);
+	/* Use connect_timeout on initial EV_WRITE stage if provided */
+	ev_tstamp start_to = (priv->connect_timeout > 0 ? priv->connect_timeout : priv->timeout);
+	rspamd_ev_watcher_start(priv->ctx->event_loop, &priv->ev, start_to);
 }
 
 return TRUE;
@@ -2634,3 +2747,81 @@ void rspamd_http_connection_disable_encryption(struct rspamd_http_connection *co
 		priv->flags &= ~RSPAMD_HTTP_CONN_FLAG_ENCRYPTED;
 	}
 }
+
+void rspamd_http_connection_set_timeouts(struct rspamd_http_connection *conn,
+										 ev_tstamp connect_timeout,
+										 ev_tstamp ssl_timeout,
+										 ev_tstamp write_timeout,
+										 ev_tstamp read_timeout)
+{
+	struct rspamd_http_connection_private *priv = conn->priv;
+
+	if (connect_timeout > 0) {
+		priv->connect_timeout = connect_timeout;
+	}
+	if (ssl_timeout > 0) {
+		priv->ssl_timeout = ssl_timeout;
+	}
+	if (write_timeout > 0) {
+		priv->write_timeout = write_timeout;
+	}
+	if (read_timeout > 0) {
+		priv->read_timeout = read_timeout;
+	}
+}
+
+void rspamd_http_connection_set_keepalive_tuning(struct rspamd_http_connection *conn,
+												 double connection_ttl,
+												 double idle_timeout,
+												 unsigned int max_reuse)
+{
+	struct rspamd_http_connection_private *priv = conn->priv;
+
+	if (connection_ttl > 0) {
+		priv->ka_ttl_override = connection_ttl;
+	}
+	if (idle_timeout > 0) {
+		priv->ka_idle_override = idle_timeout;
+	}
+	if (max_reuse > 0) {
+		priv->ka_max_reuse_override = max_reuse;
+	}
+}
+
+void rspamd_http_connection_keepalive_note_put(struct rspamd_http_connection *conn,
+											   double now_ts)
+{
+	struct rspamd_http_connection_private *priv = conn->priv;
+	priv->last_used_ts = now_ts;
+}
+
+void rspamd_http_connection_keepalive_note_reuse(struct rspamd_http_connection *conn)
+{
+	struct rspamd_http_connection_private *priv = conn->priv;
+	priv->reuse_count++;
+}
+
+gboolean rspamd_http_connection_keepalive_is_valid(struct rspamd_http_connection *conn,
+												   double now_ts,
+												   double default_ttl,
+												   unsigned int default_max_reuse)
+{
+	struct rspamd_http_connection_private *priv = conn->priv;
+	double ttl = (priv->ka_ttl_override > 0 ? priv->ka_ttl_override : default_ttl);
+	unsigned int max_reuse = (priv->ka_max_reuse_override > 0 ? priv->ka_max_reuse_override : default_max_reuse);
+
+	if (ttl > 0 && now_ts - priv->created_ts > ttl) {
+		return FALSE;
+	}
+	if (max_reuse > 0 && priv->reuse_count >= max_reuse) {
+		return FALSE;
+	}
+	return TRUE;
+}
+
+double rspamd_http_connection_keepalive_idle_timeout(struct rspamd_http_connection *conn,
+													 double default_idle)
+{
+	struct rspamd_http_connection_private *priv = conn->priv;
+	return (priv->ka_idle_override > 0 ? priv->ka_idle_override : default_idle);
+}
