@@ -1,7 +1,7 @@
 #ifndef RSPAMD_LUA_DNS_H
 #define RSPAMD_LUA_DNS_H
 
-typedef struct lua_State lua_State;
+struct lua_State;
 struct rdns_reply;
 
 /**
@@ -10,7 +10,6 @@ struct rdns_reply;
  * @param L
  * @param reply
  */
-void
-lua_push_dns_reply (lua_State *L, const struct rdns_reply *reply);
+void lua_push_dns_reply(struct lua_State *L, const struct rdns_reply *reply);
 
-#endif //RSPAMD_LUA_DNS_H
+#endif
